@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -17,5 +17,5 @@ mod reactor;
 mod ws;
 
 pub use error::{ParseError, ReactorError, WebSocketError};
-pub use reactor::{Igniter, Reactor};
+pub use reactor::{Igniter, Reactor, ReactorShutdownSignaler};
 pub use ws::{ParseBytes, WebSocketClient, WsResponse};
