@@ -42,10 +42,10 @@ use crate::transport::{Connection, RecvError};
 use self::handlers::AuthorizationDispatchBuilder;
 use self::pool::{ThreadPool, ThreadPoolBuilder};
 #[cfg(any(feature = "trust-authorization", feature = "challenge-authorization"))]
-pub(crate) use self::state_machine::AuthorizationLocalAction;
+pub(crate) use self::state_machine::AuthorizationInitiatingAction;
 pub(crate) use self::state_machine::{
-    AuthorizationActionError, AuthorizationLocalState, AuthorizationManagerStateMachine,
-    AuthorizationRemoteAction, AuthorizationRemoteState, Identity,
+    AuthorizationAcceptingAction, AuthorizationAcceptingState, AuthorizationActionError,
+    AuthorizationInitiatingState, AuthorizationManagerStateMachine, Identity,
 };
 
 const AUTHORIZATION_THREAD_POOL_SIZE: usize = 8;
@@ -55,13 +55,16 @@ const AUTHORIZATION_THREAD_POOL_SIZE: usize = 8;
 /// separately.
 #[derive(Debug, Clone)]
 pub(crate) struct ManagedAuthorizationState {
-    // Local node state while authorizing with remote node
-    local_state: AuthorizationLocalState,
-    // Remote node state
-    remote_state: AuthorizationRemoteState,
+    // Local node state while authorizing with other node
+    initiating_state: AuthorizationInitiatingState,
+    // Accepting node state
+    accepting_state: AuthorizationAcceptingState,
 
     // Tracks whether the local node has completed authorization with the remote node
     received_complete: bool,
+
+    // Stores the local authorization used
+    local_authorization: Option<Identity>,
 }
 
 #[derive(Debug)]
@@ -201,8 +204,8 @@ impl AuthorizationConnector {
             dispatcher_builder = dispatcher_builder
                 .with_signers(&self.signers)
                 .with_nonce(&nonce)
-                .with_expected_authorization(expected_authorization.clone())
-                .with_local_authorization(local_authorization.clone())
+                .with_expected_authorization(expected_authorization)
+                .with_local_authorization(local_authorization)
                 .with_verifier(verifier)
         }
 
@@ -263,9 +266,9 @@ impl AuthorizationConnector {
                 }
 
                 if state_machine
-                    .next_local_state(
+                    .next_initiating_state(
                         &connection_id,
-                        AuthorizationLocalAction::SendAuthProtocolRequest,
+                        AuthorizationInitiatingAction::SendAuthProtocolRequest,
                     )
                     .is_err()
                 {
@@ -276,7 +279,7 @@ impl AuthorizationConnector {
                 };
             }
 
-            let authed_identity = 'main: loop {
+            let authed_identities = 'main: loop {
                 match connection.recv() {
                     Ok(bytes) => {
                         let mut msg: network::NetworkMessage =
@@ -341,15 +344,19 @@ impl AuthorizationConnector {
                 }
             };
 
-            let auth_state = if let Some(auth_identity) = authed_identity {
+            // allow unused variable if challenge-authorization is not enabled
+            #[allow(unused_variables)]
+            let auth_state = if let Some((auth_identity, local_authorization)) = authed_identities {
                 match auth_identity {
                     Identity::Trust { identity } => ConnectionAuthorizationState::Authorized {
                         connection_id,
                         connection,
                         #[cfg(feature = "challenge-authorization")]
-                        expected_authorization,
+                        expected_authorization:  ConnectionAuthorizationType::Trust {
+                            identity: identity.clone()
+                        },
                         #[cfg(feature = "challenge-authorization")]
-                        local_authorization,
+                        local_authorization: local_authorization.into(),
                         identity: ConnectionAuthorizationType::Trust { identity },
                     },
                     #[cfg(feature = "challenge-authorization")]
@@ -357,9 +364,13 @@ impl AuthorizationConnector {
                         ConnectionAuthorizationState::Authorized {
                             connection_id: connection_id.clone(),
                             connection,
-                            identity: ConnectionAuthorizationType::Challenge { public_key },
-                            expected_authorization,
-                            local_authorization,
+                            identity: ConnectionAuthorizationType::Challenge {
+                                public_key: public_key.clone()
+                            },
+                            expected_authorization: ConnectionAuthorizationType::Challenge {
+                                public_key
+                            },
+                            local_authorization: local_authorization.into()
                         }
                     }
                 }
@@ -423,11 +434,17 @@ impl ManagedAuthorizations {
         }
     }
 
-    fn take_connection_identity(&mut self, connection_id: &str) -> Option<Identity> {
+    fn take_connection_identity(&mut self, connection_id: &str) -> Option<(Identity, Identity)> {
         self.states.remove(connection_id).and_then(|managed_state| {
-            match managed_state.remote_state {
-                AuthorizationRemoteState::Done(identity) => Some(identity),
-                _ => None,
+            if let Some(local_authorization) = managed_state.local_authorization {
+                match managed_state.accepting_state {
+                    AuthorizationAcceptingState::Done(identity) => {
+                        Some((identity, local_authorization))
+                    }
+                    _ => None,
+                }
+            } else {
+                None
             }
         })
     }
@@ -435,13 +452,16 @@ impl ManagedAuthorizations {
     fn is_complete(&self, connection_id: &str) -> Option<bool> {
         self.states.get(connection_id).map(|managed_state| {
             matches!(
-                (&managed_state.local_state, &managed_state.remote_state),
                 (
-                    AuthorizationLocalState::AuthorizedAndComplete,
-                    AuthorizationRemoteState::Done(_),
+                    &managed_state.initiating_state,
+                    &managed_state.accepting_state
+                ),
+                (
+                    AuthorizationInitiatingState::AuthorizedAndComplete,
+                    AuthorizationAcceptingState::Done(_),
                 ) | (
-                    AuthorizationLocalState::Unauthorized,
-                    AuthorizationRemoteState::Unauthorized
+                    AuthorizationInitiatingState::Unauthorized,
+                    AuthorizationAcceptingState::Unauthorized
                 )
             )
         })
@@ -466,9 +486,9 @@ pub enum ConnectionAuthorizationState {
         connection: Box<dyn Connection>,
         // information required if reconnect needs to be attempted
         #[cfg(feature = "challenge-authorization")]
-        expected_authorization: Option<ConnectionAuthorizationType>,
+        expected_authorization: ConnectionAuthorizationType,
         #[cfg(feature = "challenge-authorization")]
-        local_authorization: Option<ConnectionAuthorizationType>,
+        local_authorization: ConnectionAuthorizationType,
     },
     Unauthorized {
         connection_id: String,
@@ -496,6 +516,18 @@ impl std::fmt::Debug for ConnectionAuthorizationState {
     }
 }
 
+impl From<Identity> for ConnectionAuthorizationType {
+    fn from(identity: Identity) -> Self {
+        match identity {
+            Identity::Trust { identity } => ConnectionAuthorizationType::Trust { identity },
+            #[cfg(feature = "challenge-authorization")]
+            Identity::Challenge { public_key } => {
+                ConnectionAuthorizationType::Challenge { public_key }
+            }
+        }
+    }
+}
+
 #[cfg(test)]
 pub(in crate::network) mod tests {
     use super::*;
