@@ -31,30 +31,31 @@ pub use r#loop::{
     DispatchMessageReceiver, DispatchMessageSender,
 };
 
-use crate::peer::PeerAuthorizationToken;
+use crate::error::InternalError;
+use crate::peer::PeerTokenPair;
 
 /// A wrapper for a PeerId.
 ///
 /// This type constrains a dispatcher to peer-specific messages
 #[derive(Debug, Clone, PartialEq)]
-pub struct PeerId(PeerAuthorizationToken);
+pub struct PeerId(PeerTokenPair);
 
 impl std::ops::Deref for PeerId {
-    type Target = PeerAuthorizationToken;
+    type Target = PeerTokenPair;
 
     fn deref(&self) -> &Self::Target {
         &self.0
     }
 }
 
-impl From<PeerAuthorizationToken> for PeerId {
-    fn from(p: PeerAuthorizationToken) -> PeerId {
+impl From<PeerTokenPair> for PeerId {
+    fn from(p: PeerTokenPair) -> PeerId {
         PeerId(p)
     }
 }
 
-impl From<PeerId> for PeerAuthorizationToken {
-    fn from(p: PeerId) -> PeerAuthorizationToken {
+impl From<PeerId> for PeerTokenPair {
+    fn from(p: PeerId) -> PeerTokenPair {
         p.0
     }
 }
@@ -191,7 +192,7 @@ impl FromMessageBytes for RawBytes {
 /// Dispatch Errors
 ///
 /// These errors may occur when handling a dispatched message.
-#[derive(Debug, PartialEq)]
+#[derive(Debug)]
 pub enum DispatchError {
     /// An error occurred during message deserialization.
     DeserializationError(String),
@@ -205,6 +206,8 @@ pub enum DispatchError {
     HandleError(String),
     /// if no network sender is set
     MissingNetworkSender,
+    ///  An internal error occurred while a handler was executing.
+    InternalError(InternalError),
 }
 
 impl std::error::Error for DispatchError {}
@@ -224,6 +227,7 @@ impl std::fmt::Display for DispatchError {
             }
             DispatchError::HandleError(msg) => write!(f, "unable to handle message: {}", msg),
             DispatchError::MissingNetworkSender => write!(f, "missing network sender"),
+            DispatchError::InternalError(msg) => write!(f, "{}", msg),
         }
     }
 }
@@ -398,6 +402,7 @@ mod tests {
 
     use protobuf::Message;
 
+    use crate::peer::PeerAuthorizationToken;
     use crate::protos::network::{NetworkEcho, NetworkMessageType};
 
     /// Verify that messages can be dispatched to handlers via the trait.
@@ -421,14 +426,18 @@ mod tests {
         outgoing_message.set_payload(b"test_dispatcher".to_vec());
         let outgoing_message_bytes = outgoing_message.write_to_bytes().unwrap();
 
-        assert_eq!(
-            Ok(()),
-            dispatcher.dispatch(
-                PeerAuthorizationToken::from_peer_id("TestPeer").into(),
+        assert!(dispatcher
+            .dispatch(
+                PeerTokenPair::new(
+                    PeerAuthorizationToken::from_peer_id("TestPeer").into(),
+                    #[cfg(feature = "challenge-authorization")]
+                    PeerAuthorizationToken::from_peer_id("MyID").into(),
+                )
+                .into(),
                 &NetworkMessageType::NETWORK_ECHO,
                 outgoing_message_bytes
             )
-        );
+            .is_ok());
 
         assert_eq!(
             vec!["test_dispatcher".to_string()],
@@ -459,14 +468,18 @@ mod tests {
             outgoing_message.set_payload(b"thread_echo".to_vec());
             let outgoing_message_bytes = outgoing_message.write_to_bytes().unwrap();
 
-            assert_eq!(
-                Ok(()),
-                dispatcher.dispatch(
-                    PeerAuthorizationToken::from_peer_id("TestPeer").into(),
+            assert!(dispatcher
+                .dispatch(
+                    PeerTokenPair::new(
+                        PeerAuthorizationToken::from_peer_id("TestPeer"),
+                        #[cfg(feature = "challenge-authorization")]
+                        PeerAuthorizationToken::from_peer_id("MyID"),
+                    )
+                    .into(),
                     &NetworkMessageType::NETWORK_ECHO,
                     outgoing_message_bytes
                 )
-            );
+                .is_ok());
         })
         .join()
         .unwrap();
